@@ -17,8 +17,7 @@
 #
 
 import collections
-
-from .item import ConfigObjectItem
+import inspect
 
 class ConfigObject(object):
     """Base class for objects that use git-config as a backing store.  Specified
@@ -30,7 +29,6 @@ class ConfigObject(object):
                  project_section,
                  subsection,
                  ident,
-                 configitems,
                  **kwargs):
         """ConfigObject construction.  This should be treated as a private method and
         all construction should occur through the get method.
@@ -43,9 +41,6 @@ class ConfigObject(object):
 
         ident: The name of this specific ConfigObject.
 
-        configitems: A list of ConfigObjectItem describing members of the config
-                     section.
-
         **kwargs: Keyword arguments of property values to set upon construction.
 
         """
@@ -53,12 +48,25 @@ class ConfigObject(object):
         self._project_section = project_section
         self._subsection = subsection
         self._ident = ident
-        self._configitems = configitems
         self._section = ConfigObject._get_full_section(self._project_section,
                                                        self._subsection,
                                                        self._ident)
-        self._set_defaults(configitems)
-        self._init_from_dict(configitems, kwargs)
+        self._init_from_dict(kwargs)
+
+    def __setattr__(self, name, value):
+        if name.startswith('_'):
+            super().__setattr__(name, value)
+            return
+
+        # The git config can only handle string values.
+        assert(isinstance(value, str))
+        self._set_item(name, value)
+
+    def __delattr__(self, name):
+        if name.startswith('_'):
+            super().__delattr__(name)
+            return
+        self.rm_items(name)
 
     @classmethod
     def get(cls,
@@ -66,7 +74,6 @@ class ConfigObject(object):
             project_section,
             subsection,
             ident,
-            configitems,
             **kwargs):
         """Factory to construct ConfigObjects.
 
@@ -80,9 +87,6 @@ class ConfigObject(object):
 
         ident: The name of this specific ConfigObject.
 
-        configitems: A list of ConfigObjectItem describing members of the config
-                     section.
-
         **kwargs: Keyword arguments of property values to set upon construction.
 
         """
@@ -94,12 +98,12 @@ class ConfigObject(object):
         if git.has_repo():
             config_section = git.config.get_section(gitsection)
             if config_section:
-                for item in configitems:
-                    values = [value for value in config_section.iter_multival(item.key)]
-                    if len(values) == 1:
-                        values = values[0]
-                    if values:
-                        inits[item.key] = values
+                for key, item in config_section:
+                    name = key.rsplit('.', 1)[-1]
+                    if item.is_multival():
+                        inits[name] = [value for value in item.itervalues()]
+                    else:
+                        inits[name] = item.get_value()
 
         for key, value in kwargs.items():
             inits[key] = value
@@ -108,7 +112,6 @@ class ConfigObject(object):
                      project_section,
                      subsection,
                      ident,
-                     configitems,
                      **inits)
 
         return result
@@ -141,48 +144,6 @@ class ConfigObject(object):
 
         return True if git.config.get_section(gitsection) else False
 
-    @classmethod
-    def configitems(cls):
-        """Return the config items for this ConfigObject.  By default returns the
-        _configitems class attribute.  Classes without a _copnfigitems attribute
-        should override this method.
-
-        cls: The ConfigObject class to query.
-
-        """
-        return cls._configitems
-
-    @classmethod
-    def add_config_item(cls, name, default, description):
-        """Add a config item for this ConfigObject.  By default it modifies the
-        _configitems class attribute.  Classes without a _configitems atttribute
-        should override this method.
-
-        cls: The ConfigObject class to modify.
-
-        """
-        for item in cls.configitems():
-            if item.key == name:
-                return
-        cls._configitems.append(ConfigObjectItem(name, default, description))
-
-    @classmethod
-    def rm_config_item(cls, name, default, description):
-        """Remove a config item for this ConfigObject.  By default it modifies the
-        _configitems class attribute.  Classes without a _configitems atttribute
-        should override this method.
-
-        cls: The ConfigObject class to modify.
-
-        """
-        newitems = []
-        for item in cls.configitems():
-            if item.key == name:
-                continue
-            newitems.append(item)
-
-        cls._configitems = newitems
-
     @staticmethod
     def _get_full_section(section, subsection, ident):
         """Construct a full git section name by appending subsection and ident (if not
@@ -196,16 +157,6 @@ class ConfigObject(object):
             result += '.' + ident
         return result
 
-    @property
-    def section(self):
-        """Return the full git config section for this object."""
-        return self._section
-
-    @property
-    def name(self):
-        """Return the identify of this object."""
-        return self._ident
-
     @classmethod
     def get_managing_command(cls):
         """Return a command-line command key that manages the ConfigObject.  Derived
@@ -220,18 +171,45 @@ class ConfigObject(object):
         """Add a property name that reads the git config when accessed and writes the
         git config when written."""
         def fun_get(self):
-            result = [item for item in self.iter_multival(name)]
+            result = {item for item in self.iter_multival(name)}
             if not result:
                 return None
             if len(result) == 1:
-                result = result[0]
-            return result
+                return result.pop()
+            return frozenset(result)
         def fun_set(self, value):
-            self.set_item(name, value)
+            self._set_item(name, value)
         prop = property(fun_get, fun_set)
         setattr(cls, name, prop)
 
-    def set_item(self, name, value):
+    def has_item(self, name):
+        """Return whether self.<name> exists.  We don't want to use hasattr because
+        <name> is actually set on the class, not the instance.
+
+        """
+        if not isinstance(getattr(type(self), name, None), property):
+            return False
+
+        # It's not quite enough to know we have a property.  We might have a
+        # property from a previous instance but the git config entry in this
+        # instance might not be set.  This should not happen during normal
+        # operation as the command is quickly execute and the program
+        # terminates.  However, testing can have long-lived classes with many
+        # properties added to them.
+        if not self._git.has_repo():
+            return False
+
+        return self._git.config.has_item(self.get_section(), name)
+
+    def get_ident(self):
+        """Return the identifier of this object."""
+        return self._ident
+
+    def get_section(self):
+        """Return the section of this object."""
+        return self._section
+
+    def _set_item(self, name, value):
         """Set property name to value and write it to the git config."""
         if not hasattr(self.__class__, name):
             self._add_property(name)
@@ -246,7 +224,7 @@ class ConfigObject(object):
         self._git.config.rm_items(self._section, name)
         delattr(self.__class__, name)
 
-    def get_item(self, name):
+    def _get_item(self, name):
         """Get the value for property 'name.'"""
         return self._git.config.get_item(self._section, name)
 
@@ -259,7 +237,7 @@ class ConfigObject(object):
             self._add_property(name)
         self._git.config.add_item(self._section, name, value)
 
-    def _init_from_dict(self, configitems, values):
+    def _init_from_dict(self, values):
         """Set properties of the object using the mapping in dictionary valuees.  Mapped
         valuees that are sequences create multi-value keys.
 
@@ -269,21 +247,12 @@ class ConfigObject(object):
                 return False
             return isinstance(obj, collections.abc.Sequence)
 
-        for item in configitems:
-            value = values.get(item.key, None)
-            if value:
-                if issequence(value):
-                    for v in value:
-                        self.add_item(item.key, v)
-                else:
-                    self.set_item(item.key, value)
-
-    def _set_defaults(self, configitems):
-        """Query the derived class for default values of properties and set them."""
-        if self._git.has_repo():
-            for item in configitems:
-                if item.default:
-                    self.set_item(item.key, item.default)
+        for key, value in values.items():
+            if issequence(value):
+                for v in value:
+                    self.add_item(key, v)
+            else:
+                self._set_item(key, value)
 
     def iter_multival(self, name):
         """Iterate over the multiple values of a multi-value git config key."""
@@ -293,9 +262,7 @@ class ConfigObject(object):
     def __repr__(self):
         """Serialize the object as a string."""
         return str({key:value for (key, value) in
-                    map(lambda item: (item.key, [value for value in self.iter_multival(item.key)]
-                                      if hasattr(self, item.key) else None),
-                        self._configitems)})
+                    self.__dict__.items() if not key.startswith('_')})
 
     def __str__(self):
         """Serialize the object as a string."""
@@ -303,7 +270,14 @@ class ConfigObject(object):
 
     def rm(self):
         """Remove the entire section of this object from the git config."""
-        for item in self._configitems:
-            self._git.config.rm_item(self._section, item.key, '.*')
+        for key, value in self.iteritems():
+            self.rm_items(key)
         # Removing all section entries removes the section.
         #self._git.config.rm_section(self._section)
+
+    def iteritems(self):
+        """Iterate over all key, value items."""
+        for key, value in inspect.getmembers(self):
+            print(f'iteritems check {key} -> {value}')
+            if self.has_item(key):
+                yield key, value
